@@ -7,9 +7,11 @@
 # Authors:
 # - Paul Nilsson, paul.nilsson@cern.ch, 2020
 
-from pilot.util.auxiliary import get_logger
 from pilot.util.container import execute
 
+import logging
+logger = logging.getLogger(__name__)
+
 
 def get_core_count(job):
     """
@@ -42,23 +44,21 @@ def set_core_counts(job):
     :return:
     """
 
-    log = get_logger(job.jobid)
-
     if job.pgrp:
         cmd = "ps axo pgid,psr | sort | grep %d | uniq | awk '{print $1}' | grep -x %d | wc -l" % (job.pgrp, job.pgrp)
         exit_code, stdout, stderr = execute(cmd, mute=True)
-        log.debug('%s: %s' % (cmd, stdout))
+        logger.debug('%s: %s' % (cmd, stdout))
         try:
             job.actualcorecount = int(stdout)
         except Exception as e:
-            log.warning('failed to convert number of actual cores to int: %s' % e)
+            logger.warning('failed to convert number of actual cores to int: %s' % e)
         else:
-            log.debug('set number of actual cores to: %d' % job.actualcorecount)
+            logger.debug('set number of actual cores to: %d' % job.actualcorecount)
 
             # overwrite the original core count and add it to the list
             job.corecount = job.actualcorecount
             job.corecounts = add_core_count(job.actualcorecount)
-            log.debug('current core counts list: %s' % str(job.corecounts))
+            logger.debug('current core counts list: %s' % str(job.corecounts))
 
     else:
-        log.debug('payload process group not set - cannot check number of cores used by payload')
+        logger.debug('payload process group not set - cannot check number of cores used by payload')
