@@ -6,25 +6,25 @@
 # Authors:
 # - Giampaolo Rodola, g.rodola@gmail.com, 2017
 # - Daniel Drizhuk, d.drizhuk@gmail.com, 2017
+# - Paul Nilsson, Paul.Nilsson@cern.ch, 2021
 
 import os
 import collections
 
-_ntuple_diskusage = collections.namedtuple('usage', 'total used free')
+ntuple_diskusage = collections.namedtuple('usage', 'total used free')
 
 if hasattr(os, 'statvfs'):  # POSIX
     def disk_usage(path):
-        st = os.statvfs(path)
-        free = st.f_bavail * st.f_frsize
-        total = st.f_blocks * st.f_frsize
-        used = (st.f_blocks - st.f_bfree) * st.f_frsize
-        return _ntuple_diskusage(total, used, free)
-
+        stat = os.statvfs(path)
+        free = stat.f_bavail * stat.f_frsize
+        total = stat.f_blocks * stat.f_frsize
+        used = (stat.f_blocks - stat.f_bfree) * stat.f_frsize
+        return ntuple_diskusage(total, used, free)
 else:
     def disk_usage(path):
-        return _ntuple_diskusage(0, 0, 0)
+        return ntuple_diskusage(0, 0, 0)
 
 disk_usage.__doc__ = """
 Return disk usage statistics about the given path as a (total, used, free)
-namedtuple.  Values are expressed in bytes.
+namedtuple. Values are expressed in bytes.
 """
