@@ -13,7 +13,7 @@ from pilot.common.errorcodes import ErrorCodes
 from pilot.common.exception import PilotException, StageInFailure, StageOutFailure
 from pilot.util.config import config
 from pilot.util.container import execute
-from pilot.util.filehandling import copy, read_json, write_json, write_file, copy_pilot_source  #, find_executable
+from pilot.util.filehandling import copy, read_json, write_json, write_file, copy_pilot_source
 
 import logging
 logger = logging.getLogger(__name__)
@@ -151,7 +151,7 @@ def get_script_path(script):
     srcdir = environ.get('PILOT_SOURCE_DIR', '.')
     _path = path.join(srcdir, 'pilot/scripts')
     if not path.exists(_path):
-        _path = path.join(srcdir, 'pilot2')
+        _path = path.join(srcdir, 'pilot3')
         _path = path.join(_path, 'pilot/scripts')
     _path = path.join(_path, script)
     if not path.exists(_path):
